unit UBasisobjekte;

interface

uses USortElement,Sysutils;  (* Klasse TSortElement *)

type TInteger = class(TSortElement)
     private
       h_Inhalt: integer;
     public
       constructor Create(z:Integer); virtual;
       procedure SetValue (Wert: integer); virtual;
       function GetValue: integer;
       function GetText:String;override;
       (* Abstrakte Methoden berschreiben *)
       function GreaterThan(Elem: TSortElement): Boolean; override;
     end;

type TReal = class(TSortElement)
     private
       h_Inhalt: real;
     public
       constructor Create(r:Real); virtual;
       procedure SetValue (Wert: real); virtual;
       function GetValue: real;
       function GetText:String; override;
       (* Abstrakte Methoden berschreiben *)
       function GreaterThan(Elem: TSortElement): Boolean; override;
     end;

type TString = class(TSortElement)
     private
       h_Inhalt: string;
     public
       constructor Create(s:string); virtual;
       procedure SetValue (Wert: string); virtual;
       function GetValue: string;
       function GetText:String; override;
       (* Abstrakte Methoden berschreiben *)
       function GreaterThan(Elem: TSortElement): Boolean; override;
     end;

     type TBruchzahl = class(TSortElement)
     private
       Z, N: integer;
     public
       constructor Create(zp,np:Integer); virtual;
       destructor Destroy; override;
       procedure SetValue (Zaehler, Nenner: integer); virtual;
       function GetZaehler: Integer; virtual;
       function GetNenner: Integer; virtual;
       (* Abstrakte Methoden berschreiben *)
       function GetText:String; override;
       function GreaterThan (Elem: TSortElement): Boolean; override;
     end;


implementation

(********************  TInteger  ***************************************)

constructor TInteger.Create(z:Integer);
begin
  inherited Create;
  h_Inhalt := z
end;

procedure TInteger.SetValue (Wert: integer);
begin
  h_Inhalt := Wert
end;

function TInteger.GetValue: integer;
begin
  result := h_Inhalt;
end;

function TInteger.GetText: string;
begin
  result := IntToStr(h_Inhalt);
end;

function TInteger.GreaterThan(Elem: TSortElement): Boolean;
begin
  GreaterThan := (Self.GetValue > TInteger(Elem).GetValue)
end;

(********************  TReal  ***************************************)

constructor TReal.Create(r:Real);
begin
  inherited Create;
  h_Inhalt := r
end;

procedure TReal.SetValue (Wert: Real);
begin
  h_Inhalt := Wert
end;

function TReal.GetValue: Real;
begin
  result := h_Inhalt;
end;

function TReal.GetText: string;
begin
  result := FloatToStr(h_Inhalt);
end;

function TReal.GreaterThan(Elem: TSortElement): Boolean;
begin
  GreaterThan := (Self.GetValue > TReal(Elem).GetValue)
end;

(********************  TString  ***************************************)

constructor TString.Create(s:String);
begin
  inherited Create;
  h_Inhalt := s
end;

procedure TString.SetValue (Wert: String);
begin
  h_Inhalt := Wert
end;

function TString.GetValue: String;
begin
  result := h_Inhalt;
end;

function TString.GetText: string;
begin
  result := h_Inhalt;
end;

function TString.GreaterThan(Elem: TSortElement): Boolean;
begin
  GreaterThan := (Self.GetValue > TString(Elem).GetValue)
end;

(****************************  TBruchzahl  ***********************************)

constructor TBruchzahl.Create(zp,np:Integer);
begin
  inherited Create;
  Z := ZP;
  N := NP
end;

destructor TBruchzahl.Destroy;
begin
  inherited Destroy
end;

procedure TBruchzahl.SetValue (Zaehler, Nenner: integer);
begin
  Z := Zaehler;
  N := Nenner
end;

function TBruchzahl.GetZaehler: Integer;
begin
  GetZaehler := Z
end;

function TBruchzahl.GetNenner: Integer;
begin
  GetNenner := N
end;

function TBruchzahl.GetText: String;
begin
    result:=IntToStr(Z)+'/'+IntToStr(N)
end;

function TBruchzahl.GreaterThan (Elem: TSortElement): Boolean;
begin
  GreaterThan :=   (Self.GetZaehler * TBruchzahl(Elem).GetNenner)
              > (Self.GetNenner * TBruchzahl(Elem).GetZaehler)
end;

end.
