unit UListe;
interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const MaxGroesse = 100;

type
{ --------- Objekt TListObject - Vater aller Listenobjekte--------- }
    TListObject = class (TObject)
      function Get: string; virtual;  // Anzeige der Objekte als String
    end;                            // siehe bei den Nachkommen

{ --------- Der Listenbehlter ------------------------------------- }
   TListfield = array [0..MaxGroesse -1] of TListObject;
   EIndexTooLarge = class (Exception); // beim berschreiten der Gre

{ --------- Objekt TListe -die Liste selber ------------------------ }
    TListe = class (TObject)
      Count : integer;          // hlt die aktuelle Anzahl fest
    private
     field : TListfield;       // der Behlter
     aktindex: integer;        // aktuelle Feldposition
     function GetElement(Index : integer) : TListObject;
     procedure PutElement(Index : integer; Item : TListObject);
    public
     constructor Create;       // erzeugt ein neues Objekt
     procedure Free;           // beseitigt das Objekt
     procedure First;
     procedure Last;
     procedure Fw;
     procedure Bw;
     function IsFirst: boolean;
     function IsLast: boolean;
     function IsEmpty: boolean;
     function Get: TListObject;
     procedure Add (Item: TListObject); // fgt ein Objekt hinzu
     procedure Insert (Item: TListObject; Position: integer); // fgt bei Position ein
     procedure Delete; // lscht das aktuelle Objekt
     property Items[Index: Integer]: TListObject read GetElement write PutElement; default;
 //   die Eigenschaft Items hat die Lesen-Schreiben-Eigenschaft, sie hat den Type TListObjekt
 //   die Lese-Aktion ist GetElement, die Schreibe-Aktion ist PutElement
 //   Auf die Definition einer Array-Eigenschaft kann die Direktive default folgen.
 //   Dann wird die Array-Eigenschaft zur voreingestellten Array-Eigenschaft der Klasse, d.h.
 //   anstelle auf field.items[..] kann auf field[..] zugegriffen werden.
  end;

implementation

uses UElemente;

{ $R *.DFM}

{ --------- Objekt TListObject ------------------------------------- }

function TListObject.Get: string;
begin end;

{ --------- Objekt TListe ------------------------------------------ }

constructor TListe.Create;
var i : integer;
begin
  inherited Create;
  Count := 0;                // Initialisierung
  aktindex := 0;
  for i := 0 to MaxGroesse-1 do field[i] := nil;
end;

procedure TListe.Free;
var i: integer;
begin                        // Speicherplatz freigeben
  for i := 0 to MaxGroesse-1 do field[i].Free;
  Count := -1;
  inherited Destroy;
end;

procedure TListe.First;
begin
  aktindex := 0
end;

procedure TListe.Last;
begin
  aktindex := count - 1
end;

procedure TListe.Fw;
begin
  if aktindex < (count - 1) then inc(aktindex)
end;

procedure TListe.Bw;
begin
  if aktindex > 0 then dec(aktindex)
end;

function TListe.IsFirst: boolean;
begin
  result := aktindex = 0
end;

function TListe.IsLast: boolean;
begin
  result := aktindex = count - 1
end;

function TListe.IsEmpty: boolean;
begin
  result := aktindex = -1
end;

function TListe.GetElement(Index : integer) : TListObject;
begin
  if Index > Count then raise EIndexTooLarge.CreateFmt ('Der Index der Liste berschreitet das Maximum (%d)',[Index]);
  result := field[Index];
end;

procedure TListe.PutElement(Index : integer; Item : TListObject);
begin
  if Index >= MaxGroesse then raise EIndexTooLarge.CreateFmt ('Der Index der Liste berschreitet das Maximum (%d)',[Index]);
  field[Index] := Item;
end;

function TListe.Get: TListObject;
begin
  result := field[aktindex]
end;

procedure TListe.Add (Item: TListObject);
begin
  if Count = MaxGroesse then raise EIndexTooLarge.CreateFmt ('Die Liste ist voll (%d)',[Count]);
  field[Count] := Item;
  inc (Count);
end;

procedure TListe.Insert (Item: TListObject; Position: integer);
var i : integer;
begin
   if Position >= MaxGroesse then raise EIndexTooLarge.CreateFmt ('Die Liste ist voll (%d)',[Count]);
   Add(Items[Count-1]);
   for i := Count-1 downto Position+1 do
    begin          // aufrcken, um Platz zu schaffen
     Items[i] := Items[i-1];
    end;
    Items[Position] := Item;
end;

procedure TListe.Delete;
var i : integer;
begin
   Items[aktindex].Free;
   for i := aktindex to count - 2 do Items[i] := Items[i+1]; // herunterrcken
   dec(count)
end;

end.


